@@ -1,11 +1,15 @@
 #ifndef __ASYNC_IO_H__
 #define __ASYNC_IO_H__
+
 #ifdef _SQUID_WIN32_
 #include "aio_win32.h"
 #else
 #include <aio.h>
 #endif
 
+/* for FREE* */
+#include "typedefs.h"
+
 #define MAX_ASYNCOP		128
 
 typedef enum {
