@@ -59,6 +59,9 @@
 #ifndef USE_DNSSERVERS
 #ifdef _SQUID_WIN32_
 #include "squid_windows.h"
+#define REG_TCPIP_PARA_INTERFACES "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Interfaces"
+#define REG_TCPIP_PARA "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters"
+#define REG_VXD_MSTCP "SYSTEM\\CurrentControlSet\\Services\\VxD\\MSTCP"
 #endif
 #ifndef _PATH_RESCONF
 #define _PATH_RESCONF "/etc/resolv.conf"
@@ -373,32 +376,24 @@ idnsParseWIN32SearchList(const char * Separator)
     char *token;
     HKEY hndKey;
 
-    if (RegOpenKey(HKEY_LOCAL_MACHINE,
-                   "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
-                   &hndKey) == ERROR_SUCCESS) {
+    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_TCPIP_PARA, 0, KEY_QUERY_VALUE, &hndKey) == ERROR_SUCCESS) {
         DWORD Type = 0;
         DWORD Size = 0;
         LONG Result;
-        Result =
-            RegQueryValueEx(hndKey, "Domain", NULL, &Type, NULL,
-                            &Size);
+	Result = RegQueryValueEx(hndKey, "Domain", NULL, &Type, NULL, &Size);
 
         if (Result == ERROR_SUCCESS && Size) {
             t = (char *) xmalloc(Size);
-            RegQueryValueEx(hndKey, "Domain", NULL, &Type, (LPBYTE) t,
-                            &Size);
+            RegQueryValueEx(hndKey, "Domain", NULL, &Type, (LPBYTE) t, &Size);
             debugs(78, 1, "Adding domain " << t << " from Registry");
             idnsAddPathComponent(t);
             xfree(t);
         }
-        Result =
-            RegQueryValueEx(hndKey, "SearchList", NULL, &Type, NULL,
-                            &Size);
+	Result = RegQueryValueEx(hndKey, "SearchList", NULL, &Type, NULL, &Size);
 
         if (Result == ERROR_SUCCESS && Size) {
             t = (char *) xmalloc(Size);
-            RegQueryValueEx(hndKey, "SearchList", NULL, &Type, (LPBYTE) t,
-                            &Size);
+	    RegQueryValueEx(hndKey, "SearchList", NULL, &Type, (LPBYTE) t, &Size);
             token = strtok(t, Separator);
 
             while (token) {
@@ -430,20 +425,15 @@ idnsParseWIN32Registry(void)
     case _WIN_OS_WINNT:
         /* get nameservers from the Windows NT registry */
 
-        if (RegOpenKey(HKEY_LOCAL_MACHINE,
-                       "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
-                       &hndKey) == ERROR_SUCCESS) {
+	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_TCPIP_PARA, 0, KEY_QUERY_VALUE, &hndKey) == ERROR_SUCCESS) {
             DWORD Type = 0;
             DWORD Size = 0;
             LONG Result;
-            Result =
-                RegQueryValueEx(hndKey, "DhcpNameServer", NULL, &Type, NULL,
-                                &Size);
+	    Result = RegQueryValueEx(hndKey, "DhcpNameServer", NULL, &Type, NULL, &Size);
 
             if (Result == ERROR_SUCCESS && Size) {
                 t = (char *) xmalloc(Size);
-                RegQueryValueEx(hndKey, "DhcpNameServer", NULL, &Type, (LPBYTE) t,
-                                &Size);
+                RegQueryValueEx(hndKey, "DhcpNameServer", NULL, &Type, (LPBYTE) t, &Size);
                 token = strtok(t, ", ");
 
                 while (token) {
@@ -454,8 +444,7 @@ idnsParseWIN32Registry(void)
                 xfree(t);
             }
 
-            Result =
-                RegQueryValueEx(hndKey, "NameServer", NULL, &Type, NULL, &Size);
+            Result = RegQueryValueEx(hndKey, "NameServer", NULL, &Type, NULL, &Size);
 
             if (Result == ERROR_SUCCESS && Size) {
                 t = (char *) xmalloc(Size);
@@ -484,67 +473,68 @@ idnsParseWIN32Registry(void)
     case _WIN_OS_WINNET:
 
     case _WIN_OS_WINLON:
+
+    case _WIN_OS_WIN7:
         /* get nameservers from the Windows 2000 registry */
         /* search all interfaces for DNS server addresses */
 
-        if (RegOpenKey(HKEY_LOCAL_MACHINE,
-                       "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Interfaces",
-                       &hndKey) == ERROR_SUCCESS) {
+	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_TCPIP_PARA_INTERFACES, 0, KEY_READ, &hndKey) == ERROR_SUCCESS) {
             int i;
-            char keyname[255];
+	    int MaxSubkeyLen;
+	    DWORD InterfacesCount;
+	    char *keyname;
+	    FILETIME ftLastWriteTime;
 
-            for (i = 0; i < 10; i++) {
-                if (RegEnumKey(hndKey, i, (char *) &keyname,
-                               255) == ERROR_SUCCESS) {
-                    char newkeyname[255];
-                    strcpy(newkeyname,
-                           "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Interfaces\\");
-                    strcat(newkeyname, keyname);
+	    if (RegQueryInfoKey(hndKey, NULL, NULL, NULL, &InterfacesCount, &MaxSubkeyLen, NULL, NULL, NULL, NULL, NULL, NULL) == ERROR_SUCCESS) {
+		keyname = (char *) xmalloc(++MaxSubkeyLen);
+		for (i = 0; i < (int) InterfacesCount; i++) {
+		    int j;
+		    j = MaxSubkeyLen;
+		    if (RegEnumKeyEx(hndKey, i, keyname, &j, NULL, NULL, NULL, &ftLastWriteTime) == ERROR_SUCCESS) {
+			char *newkeyname;
+			newkeyname = (char *) xmalloc(sizeof(REG_TCPIP_PARA_INTERFACES) + j + 2);
+			strcpy(newkeyname, REG_TCPIP_PARA_INTERFACES);
+			strcat(newkeyname, "\\");
+			strcat(newkeyname, keyname);
+			if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, newkeyname, 0, KEY_QUERY_VALUE, &hndKey2) == ERROR_SUCCESS) {
+			    DWORD Type = 0;
+			    DWORD Size = 0;
+			    LONG Result;
+			    Result = RegQueryValueEx(hndKey2, "DhcpNameServer", NULL, &Type, NULL, &Size);
+			    if (Result == ERROR_SUCCESS && Size) {
+				t = (char *) xmalloc(Size);
+				RegQueryValueEx(hndKey2, "DhcpNameServer", NULL, &Type, t, &Size);
+				token = strtok(t, ", ");
+				while (token) {
+                                    debugs(78, 1, "Adding DHCP nameserver " << token << " from Registry");
+				    idnsAddNameserver(token);
+				    token = strtok(NULL, ", ");
+				}
+				xfree(t);
+ 			    }
 
-                    if (RegOpenKey(HKEY_LOCAL_MACHINE, newkeyname,
-                                   &hndKey2) == ERROR_SUCCESS) {
-                        DWORD Type = 0;
-                        DWORD Size = 0;
-                        LONG Result;
-                        Result =
-                            RegQueryValueEx(hndKey2, "DhcpNameServer", NULL,
-                                            &Type, NULL, &Size);
+			    Result = RegQueryValueEx(hndKey2, "NameServer", NULL, &Type, NULL, &Size);
+			    if (Result == ERROR_SUCCESS && Size) {
+				t = (char *) xmalloc(Size);
+				RegQueryValueEx(hndKey2, "NameServer", NULL, &Type, t, &Size);
+				token = strtok(t, ", ");
+				while (token) {
+				    debugs(78, 1, "Adding nameserver " << token << " from Registry");
+				    idnsAddNameserver(token);
+				    token = strtok(NULL, ", ");
+				}
 
-                        if (Result == ERROR_SUCCESS && Size) {
-                            t = (char *) xmalloc(Size);
-                            RegQueryValueEx(hndKey2, "DhcpNameServer", NULL,
-                                            &Type, (LPBYTE) t, &Size);
-                            token = strtok(t, ", ");
-
-                            while (token) {
-                                debugs(78, 1, "Adding DHCP nameserver " << token << " from Registry");
-                                idnsAddNameserver(token);
-                                token = strtok(NULL, ", ");
+				xfree(t);
                             }
-                            xfree(t);
+
+                            RegCloseKey(hndKey2);
                         }
 
-                        Result =
-                            RegQueryValueEx(hndKey2, "NameServer", NULL, &Type,
-                                            NULL, &Size);
-
-                        if (Result == ERROR_SUCCESS && Size) {
-                            t = (char *) xmalloc(Size);
-                            RegQueryValueEx(hndKey2, "NameServer", NULL, &Type,
-                                            (LPBYTE) t, &Size);
-                            token = strtok(t, ", ");
-
-                            while (token) {
-                                debugs(78, 1, "Adding nameserver " << token << " from Registry");
-                                idnsAddNameserver(token);
-                                token = strtok(NULL, ", ");
-                            }
-                            xfree(t);
-                        }
-
-                        RegCloseKey(hndKey2);
+			xfree(newkeyname);
                     }
                 }
+
+		xfree(keyname);
             }
 
             RegCloseKey(hndKey);
@@ -561,14 +551,11 @@ idnsParseWIN32Registry(void)
     case _WIN_OS_WINME:
         /* get nameservers from the Windows 9X registry */
 
-        if (RegOpenKey(HKEY_LOCAL_MACHINE,
-                       "SYSTEM\\CurrentControlSet\\Services\\VxD\\MSTCP",
-                       &hndKey) == ERROR_SUCCESS) {
+	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_VXD_MSTCP, 0, KEY_QUERY_VALUE, &hndKey) == ERROR_SUCCESS) {
             DWORD Type = 0;
             DWORD Size = 0;
             LONG Result;
-            Result =
-                RegQueryValueEx(hndKey, "NameServer", NULL, &Type, NULL, &Size);
+	    Result = RegQueryValueEx(hndKey, "NameServer", NULL, &Type, NULL, &Size);
 
             if (Result == ERROR_SUCCESS && Size) {
                 t = (char *) xmalloc(Size);
@@ -677,9 +664,9 @@ idnsSentQueryVC(int fd, char *buf, size_t size, comm_err_t flag, int xerrno, voi
 
     if (flag == COMM_ERR_CLOSING)
         return;
-    
+
     if (fd_table[fd].closing())
-	return;
+        return;
 
     if (flag != COMM_OK || size <= 0) {
         comm_close(fd);
@@ -1343,34 +1330,20 @@ idnsInit(void)
     if (DnsSocket < 0) {
         int port;
 
-        IpAddress addr;
+        IpAddress addr; // since we don't want to alter Config.Addrs.udp_* and dont have one of our own.
 
         if (!Config.Addrs.udp_outgoing.IsNoAddr())
             addr = Config.Addrs.udp_outgoing;
         else
             addr = Config.Addrs.udp_incoming;
 
-        DnsSocket = comm_open(SOCK_DGRAM,
+        debugs(78, 2, "idnsInit: attempt open DNS socket to: " << addr);
+        DnsSocket = comm_open_listener(SOCK_DGRAM,
                               IPPROTO_UDP,
                               addr,
                               COMM_NONBLOCKING,
                               "DNS Socket");
 
-        debugs(78, 2, "idnsInit: attempt open DNS socket to: " << addr);
-
-#if USE_IPV6
-        if ( DnsSocket < 0 && addr.IsIPv6() && addr.SetIPv4() ) {
-            /* attempt to open this IPv4-only. */
-            DnsSocket = comm_open(SOCK_DGRAM,
-                                  IPPROTO_UDP,
-                                  addr,
-                                  COMM_NONBLOCKING,
-                                  "DNS Socket");
-
-            debugs(78, 2, "idnsInit: attempt open DNS socket to: " << addr);
-        }
-#endif
-
         if (DnsSocket < 0)
             fatal("Could not create a DNS socket");
 
