@@ -664,7 +664,7 @@ icpConnectionsOpen(void)
 
     addr = Config.Addrs.udp_incoming;
     addr.SetPort(port);
-    theInIcpConnection = comm_open(SOCK_DGRAM,
+    theInIcpConnection = comm_open_listener(SOCK_DGRAM,
                                    IPPROTO_UDP,
                                    addr,
                                    COMM_NONBLOCKING,
@@ -690,7 +690,7 @@ icpConnectionsOpen(void)
     if ( !addr.IsNoAddr() ) {
         enter_suid();
         addr.SetPort(port);
-        theOutIcpConnection = comm_open(SOCK_DGRAM,
+        theOutIcpConnection = comm_open_listener(SOCK_DGRAM,
                                         IPPROTO_UDP,
                                         addr,
                                         COMM_NONBLOCKING,
