@@ -447,6 +447,8 @@ struct SquidConfig {
         int WIN32_IpAddrChangeMonitor;
     } onoff;
 
+    int forward_max_tries;
+
     class ACL *aclList;
 
     struct {
@@ -462,8 +464,9 @@ struct SquidConfig {
 
         acl_access *snmp;
 #endif
-
+#if HTTP_VIOLATIONS
         acl_access *brokenPosts;
+#endif
 #if USE_IDENT
 
         acl_access *identLookup;
@@ -837,8 +840,8 @@ struct peer {
         int counts[ICP_END+1];
         u_short port;
     } icp;
-#if USE_HTCP
 
+#if USE_HTCP
     struct {
         double version;
         int counts[2];
@@ -929,6 +932,7 @@ struct peer {
 
     char *login;		/* Proxy authorization */
     time_t connect_timeout;
+    int connect_fail_limit;
     int max_conn;
     char *domain;		/* Forced domain */
 #if USE_SSL
@@ -1085,6 +1089,7 @@ struct _refresh_t {
         unsigned int ignore_reload:1;
         unsigned int ignore_no_cache:1;
         unsigned int ignore_no_store:1;
+        unsigned int ignore_must_revalidate:1;
         unsigned int ignore_private:1;
         unsigned int ignore_auth:1;
 #endif
