@@ -1,6 +1,6 @@
 
 /*
- * $Id: tunnel.cc,v 1.177 2008/01/20 08:54:28 amosjeffries Exp $
+ * $Id$
  *
  * DEBUG: section 26    Secure Sockets Layer Proxy
  * AUTHOR: Duane Wessels
@@ -39,7 +39,7 @@
 #include "fde.h"
 #include "comm.h"
 #include "client_side_request.h"
-#include "ACLChecklist.h"
+#include "acl/FilledChecklist.h"
 #if DELAY_POOLS
 #include "DelayId.h"
 #endif
@@ -114,6 +114,8 @@ private:
     void writeServerDone(char *buf, size_t len, comm_err_t flag, int xerrno);
 };
 
+#define fd_closed(fd) (fd == -1 || fd_table[fd].closing())
+
 static const char *const conn_established = "HTTP/1.0 200 Connection established\r\n\r\n";
 
 static CNCB tunnelConnectDone;
@@ -258,11 +260,8 @@ TunnelStateData::Connection::error(int const xerrno)
     /* XXX fixme xstrerror and xerrno... */
     errno = xerrno;
 
-    if (xerrno == COMM_ERR_CLOSING)
-        return;
-
-    debugs(50, debugLevelForError(xerrno), "tunnelReadServer: FD " << fd() <<
-           ": read failure: " << xstrerror());
+    debugs(50, debugLevelForError(xerrno), "TunnelStateData::Connection::error: FD " << fd() <<
+           ": read/write failure: " << xstrerror());
 
     if (!ignoreErrno(xerrno))
         comm_close(fd());
@@ -309,16 +308,16 @@ TunnelStateData::copy (size_t len, comm_err_t errcode, int xerrno, Connection &f
     cbdataInternalLock(this);	/* ??? should be locked by the caller... */
 
     /* Bump the server connection timeout on any activity */
-    if (server.fd() != -1)
+    if (!fd_closed(server.fd()))
         commSetTimeout(server.fd(), Config.Timeout.read, tunnelTimeout, this);
 
     if (len < 0 || errcode)
         from.error (xerrno);
-    else if (len == 0 || to.fd() == -1) {
+    else if (len == 0 || fd_closed(to.fd())) {
         comm_close(from.fd());
         /* Only close the remote end if we've finished queueing data to it */
 
-        if (from.len == 0 && to.fd() != -1) {
+        if (from.len == 0 && !fd_closed(to.fd()) ) {
             comm_close(to.fd());
         }
     } else if (cbdataReferenceValid(this))
@@ -343,6 +342,9 @@ TunnelStateData::writeServerDone(char *buf, size_t len, comm_err_t flag, int xer
 {
     debugs(26, 3, "tunnelWriteServer: FD " << server.fd() << ", " << len << " bytes written");
 
+    if(flag == COMM_ERR_CLOSING)
+        return;
+
     /* Error? */
     if (len < 0 || flag != COMM_OK) {
         server.error(xerrno); // may call comm_close
@@ -361,7 +363,7 @@ TunnelStateData::writeServerDone(char *buf, size_t len, comm_err_t flag, int xer
     client.dataSent(len);
 
     /* If the other end has closed, so should we */
-    if (client.fd() == -1) {
+    if (fd_closed(client.fd())) {
         comm_close(server.fd());
         return;
     }
@@ -401,6 +403,9 @@ TunnelStateData::writeClientDone(char *buf, size_t len, comm_err_t flag, int xer
 {
     debugs(26, 3, "tunnelWriteClient: FD " << client.fd() << ", " << len << " bytes written");
 
+    if(flag == COMM_ERR_CLOSING)
+        return;
+
     /* Error? */
     if (len < 0 || flag != COMM_OK) {
         client.error(xerrno); // may call comm_close
@@ -418,7 +423,7 @@ TunnelStateData::writeClientDone(char *buf, size_t len, comm_err_t flag, int xer
     server.dataSent(len);
 
     /* If the other end has closed, so should we */
-    if (server.fd() == -1) {
+    if (fd_closed(server.fd())) {
         comm_close(client.fd());
         return;
     }
@@ -447,7 +452,7 @@ tunnelTimeout(int fd, void *data)
 void
 TunnelStateData::Connection::closeIfOpen()
 {
-    if (fd() != -1)
+    if (!fd_closed(fd()))
         comm_close(fd());
 }
 
@@ -537,10 +542,10 @@ tunnelErrorComplete(int fdnotused, void *data, size_t sizenotused)
     /* temporary lock to save our own feets (comm_close -> tunnelClientClosed -> Free) */
     cbdataInternalLock(tunnelState);
 
-    if (tunnelState->client.fd() > -1)
+    if (!fd_closed(tunnelState->client.fd()))
         comm_close(tunnelState->client.fd());
 
-    if (tunnelState->server.fd() > -1)
+    if (fd_closed(tunnelState->server.fd()))
         comm_close(tunnelState->server.fd());
 
     cbdataInternalUnlock(tunnelState);
@@ -611,16 +616,14 @@ tunnelStart(ClientHttpRequest * http, int64_t * size_ptr, int *status_ptr)
      * be allowed.  yuck, I know.
      */
 
-    if (!request->client_addr.IsNoAddr()) {
+    if (!request->client_addr.IsNoAddr() && Config.accessList.miss) {
         /*
          * Check if this host is allowed to fetch MISSES from us (miss_access)
+         * default is to allow.
          */
-        ACLChecklist ch;
+        ACLFilledChecklist ch(Config.accessList.miss, request, NULL);
         ch.src_addr = request->client_addr;
         ch.my_addr = request->my_addr;
-        ch.request = HTTPMSGLOCK(request);
-        ch.accessList = cbdataReference(Config.accessList.miss);
-        /* cbdataReferenceDone() happens in either fastCheck() or ~ACLCheckList */
         answer = ch.fastCheck();
 
         if (answer == 0) {
@@ -635,11 +638,15 @@ tunnelStart(ClientHttpRequest * http, int64_t * size_ptr, int *status_ptr)
     statCounter.server.all.requests++;
     statCounter.server.other.requests++;
     /* Create socket. */
-    IPAddress temp = getOutgoingAddr(request,NULL);
+    IpAddress temp = getOutgoingAddr(request,NULL);
+    int flags = COMM_NONBLOCKING;
+    if (request->flags.spoof_client_ip) {
+        flags |= COMM_TRANSPARENT;
+    }
     sock = comm_openex(SOCK_STREAM,
                        IPPROTO_TCP,
                        temp,
-                       COMM_NONBLOCKING,
+                       flags,
                        getOutgoingTOS(request),
                        url);
 
@@ -795,7 +802,7 @@ TunnelStateData::Connection::fd(int const newFD)
 bool
 TunnelStateData::noConnections() const
 {
-    return (server.fd() == -1) && (client.fd() == -1);
+    return fd_closed(server.fd()) && fd_closed(client.fd());
 }
 
 #if DELAY_POOLS
