@@ -1,6 +1,6 @@
 
 /*
- * $Id: wccp.cc,v 1.45 2007/12/14 23:11:48 amosjeffries Exp $
+ * $Id$
  *
  * DEBUG: section 80    WCCP Support
  * AUTHOR: Glenn Chisholm
@@ -147,7 +147,7 @@ wccpConnectionOpen(void)
 
     Config.Wccp.address.SetPort(WCCP_PORT);
 
-    theWccpConnection = comm_open(SOCK_DGRAM,
+    theWccpConnection = comm_open_listener(SOCK_DGRAM,
                                   IPPROTO_UDP,
                                   Config.Wccp.address,
                                   COMM_NONBLOCKING,
